"""
py-reddit: Reddit and python, made simple.
~~~~~~~~~~~~~~~~~~~
A basic wrapper for managing reddit submissions.
:copyright: (c) 2022-present KING7077
:license: MIT, see LICENSE for more details.
"""
__title__ = 'py-reddit'
__author__ = 'KING7077'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-present KING7077'
__version__ = '1.1.3'

from .reddit import *
